"""Ujan RoyBandyopadhyay
October 2022

This module provides functions for analysis of randomly colored
grids."""

from rdconfigs.grids.type_aliases import RGB

def find_centroid(grid: list[list[RGB]], color: RGB) -> tuple[float, float]:
    """Return the position of a color's center of mass in a grid.

    Arguments:
        - grid
            a list containing equal-sized lists of RGB values
        - color
            the color whose center of mass to find

    Returns:
        - a (row, col) tuple containing the position of the color's
          center of mass on the grid
    """
    coordinates = get_graph(grid, color)

    row = sum(coordinate[0] for coordinate in coordinates) / len(coordinates)
    col = sum(coordinate[1] for coordinate in coordinates) / len(coordinates)

    return (row, col)

def find_centroids(grid: list[list[RGB]]) -> dict[RGB, tuple[float, float]]:
    """Return the centers of mass of each color in a grid.

    Arguments:
        - grid
            a list containing equal-sized lists of RGB values

    Returns:
        - a dictionary whose keys are colors and values are (row, col)
          tuples containing the position of the color's center of mass
          on the grid
    """
    colors = {square for row in grid for square in row}
    return {color: find_centroid(color) for color in colors}

def get_graph(grid: list[list[RGB]], /, *colors: RGB) -> set[tuple[int, int]]:
    """Return a set containing all squares of color.

    Positional arguments:
        - grid
            a list containing equal-sized lists of RGB values
        - *colors
            the color(s) to check for

    Returns:
        - a set containing (row, col) tuples representing each square
          of the given color
    """
    return {(i, j)
            for i, row in enumerate(grid)
            for j, square in enumerate(row)
            if square in colors}



"""Ujan RoyBandyopadhyay
October 2022

Generate, display, and analyze randomly colored grids.

Example usage:
>>> frequencies = {(0, 0, 0): 3, (255, 255, 255): 6}
>>> g = generate_grid(frequencies, rows=3, cols=3)
>>> print(g)
[[(0, 0, 0), (255, 255, 255), (255, 255, 255)],
 [(255, 255, 255), (0, 0, 0), (255, 255, 255)],
 [(255, 255, 255), (255, 255, 255), (0, 0, 0)]]
>>> display(g)  # display `g` in a new window
"""

__all__ = ['display', 'find_centroid', 'find_centroids', 'generate_grid',
           'get_graph', 'RGB']

from random_tiling.grids.analysis import (
    find_centroid, find_centroids, get_graph
)
from random_tiling.grids.generate import generate_grid
from random_tiling.grids.type_aliases import RGB
from random_tiling.ui.ui_tools import display



"""Ujan RoyBandyopadhyay
October 2022

This module provides functions for displaying grids using pygame."""

import pygame as pg

from rdconfigs.grids.type_aliases import RGB

def display(grid: list[list[RGB]], *,
            square_width: int = 5, square_height: int = 5,
            border: int = 0, title: str = 'Tiles',
            border_color: RGB = (0, 0, 0)):
    """Display a grid of colors in a new window.

    Arguments:
        - grid
            a list containing equal-sized lists of RGB values

    Keyword Arguments:
        - square_width
            the width (in pixels) of each displayed square
        - square_height
            the height (in pixels) of each displayed square
        - border
            the width (in pixels) of the border drawn between squares
        - title
            the title of the display window
        - border_color
            the color of the border between squares
    """
    rows, cols = len(grid), len(grid[0])
    height, width = rows * square_height, cols * square_width

    pg.init()

    window = pg.display.set_mode((width, height))
    pg.display.set_caption(title)
    window.fill(border_color)
    fill_grid(grid, window, border=border)
    pg.display.update()

    running = True
    while running:
        for event in pg.event.get():
            if event.type == pg.QUIT:
                running = False
                break

    pg.quit()

def fill_grid(grid: list[list[RGB]], window: pg.Surface, *, border: int = 0):
    """Draw grid on window.

    Arguments:
        - grid
            a list containing equal-sized lists of RGB values
        - window
            pygame surface to draw grid onto

    Keyword Arguments:
        - border
            width (in pixels) of border to draw between squares
    """
    square_width = window.get_width() // len(grid)
    square_height = window.get_height() // len(grid[0])

    for i, row in enumerate(grid):
        for j, color in enumerate(row):
            fill_square((i, j), color, window,
                        square_width=square_width,
                        square_height=square_height,
                        border=border)

def fill_square(square: tuple[int, int], color: RGB, window: pg.Surface, *,
                square_width: int, square_height: int, border: int = 0):
    """Fill square in window with color.

    Arguments:
        - square
            coordinate of square (row, col)
        - color
            color with which to fill square
        - window
            pygame surface where square is located

    Keyword Arguments:
        - square_width
            pixel width of each square in window
        - square_height
            pixel height of each square in window
        - border
            pixel width of border between squares
    """
    row, col = square
    rect = pg.Rect(col*square_width + border,
                   row*square_height + border,
                   square_width - border,
                   square_height - border)
    window.fill(color, rect)



"""Ujan RoyBandyopadhyay
October 2022

This module provides tools for the generation of randomly colored
grids."""

import random

from rdconfigs.grids.type_aliases import RGB

def generate_grid(frequencies: dict[RGB, int],
                  rows: int, cols: int) -> list[list[RGB]]:
    """Return a grid (nested list) of RGB tuples.

    Arguments:
        - frequencies
            a dictionary whose keys are colors and values are the
            number of times each color should appear in the grid
        - rows
            the number of rows to generate
        - cols
            the number of squares to generate per row

    Raises:
        - ValueError if `sum(frequencies.values()) != rows*cols`

    Returns:
        - a list containing equal-sized lists of RGB values
    """
    grid_size = rows * cols
    if grid_size != sum(frequencies.values()):
        raise ValueError(f'cannot fill {rows}x{cols} grid '
                         f'with {sum(frequencies.values())} values')

    flat_grid = [None] * grid_size
    shuffled_squares = random.sample(range(grid_size), grid_size)
    start = 0

    for color in frequencies:
        assigned_squares = shuffled_squares[start : start+frequencies[color]]

        for square in assigned_squares:
            flat_grid[square] = color

        start += frequencies[color]

    return [flat_grid[row*cols : row*cols + cols] for row in range(rows)]
